! Do not edit this file. It is produced from the corresponding .m4 source
!
!  Copyright (C) 2013, Northwestern University and Argonne National Laboratory
!  See COPYRIGHT notice in top-level directory.
!
! $Id: getput_text.m4 1882 2014-11-21 17:44:34Z wkliao $
!


!
! Independent put APIs
!

   function nf90mpi_put_var_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*),                                      intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map

     integer                                                            :: nf90mpi_put_var_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride
 
     ! Set local arguments to default values
     localStart (:)  = 1
     localCount (1)  = LEN(values); localCount (2:) = 1
     localStride(:)  = 1

     if(present(start))  localStart (:size(start) ) = start(:)
     if(present(count))  localCount (:size(count) ) = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map)) then
       nf90mpi_put_var_text = nfmpi_put_varm_text(ncid, varid, localStart, localCount, localStride, map, values)
     else
       nf90mpi_put_var_text = nfmpi_put_vars_text(ncid, varid, localStart, localCount, localStride, values)
     end if
   end function nf90mpi_put_var_text

   function nf90mpi_get_var_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*),                                      intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map

     integer                                                            :: nf90mpi_get_var_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride
 
     ! Set local arguments to default values
     localStart (:)  = 1
     localCount (1)  = LEN(values); localCount (2:) = 1
     localStride(:)  = 1

     if(present(start))  localStart (:size(start) ) = start(:)
     if(present(count))  localCount (:size(count) ) = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map)) then
       nf90mpi_get_var_text = nfmpi_get_varm_text(ncid, varid, localStart, localCount, localStride, map, values)
     else
       nf90mpi_get_var_text = nfmpi_get_vars_text(ncid, varid, localStart, localCount, localStride, values)
     end if
   end function nf90mpi_get_var_text


!
! Collective put APIs
!

   function nf90mpi_put_var_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*),                                      intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map

     integer                                                            :: nf90mpi_put_var_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride
 
     ! Set local arguments to default values
     localStart (:)  = 1
     localCount (1)  = LEN(values); localCount (2:) = 1
     localStride(:)  = 1

     if(present(start))  localStart (:size(start) ) = start(:)
     if(present(count))  localCount (:size(count) ) = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map)) then
       nf90mpi_put_var_text_all = nfmpi_put_varm_text_all(ncid, varid, localStart, localCount, localStride, map, values)
     else
       nf90mpi_put_var_text_all = nfmpi_put_vars_text_all(ncid, varid, localStart, localCount, localStride, values)
     end if
   end function nf90mpi_put_var_text_all

   function nf90mpi_get_var_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*),                                      intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map

     integer                                                            :: nf90mpi_get_var_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride
 
     ! Set local arguments to default values
     localStart (:)  = 1
     localCount (1)  = LEN(values); localCount (2:) = 1
     localStride(:)  = 1

     if(present(start))  localStart (:size(start) ) = start(:)
     if(present(count))  localCount (:size(count) ) = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map)) then
       nf90mpi_get_var_text_all = nfmpi_get_varm_text_all(ncid, varid, localStart, localCount, localStride, map, values)
     else
       nf90mpi_get_var_text_all = nfmpi_get_vars_text_all(ncid, varid, localStart, localCount, localStride, values)
     end if
   end function nf90mpi_get_var_text_all




   function nf90mpi_put_var_1D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_1D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 1

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_1D_text = &
          nfmpi_put_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1))
     else
       nf90mpi_put_var_1D_text = &
          nfmpi_put_vars_text(ncid, varid, localStart, localCount, localStride, values(1))
     end if
   end function nf90mpi_put_var_1D_text

   function nf90mpi_put_var_2D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_2D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 2

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_2D_text = &
          nfmpi_put_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1))
     else
       nf90mpi_put_var_2D_text = &
          nfmpi_put_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1))
     end if
   end function nf90mpi_put_var_2D_text

   function nf90mpi_put_var_3D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_3D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 3

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_3D_text = &
          nfmpi_put_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1))
     else
       nf90mpi_put_var_3D_text = &
          nfmpi_put_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1))
     end if
   end function nf90mpi_put_var_3D_text

   function nf90mpi_put_var_4D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_4D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 4

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_4D_text = &
          nfmpi_put_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1))
     else
       nf90mpi_put_var_4D_text = &
          nfmpi_put_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1))
     end if
   end function nf90mpi_put_var_4D_text

   function nf90mpi_put_var_5D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_5D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 5

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_5D_text = &
          nfmpi_put_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1))
     else
       nf90mpi_put_var_5D_text = &
          nfmpi_put_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1))
     end if
   end function nf90mpi_put_var_5D_text

   function nf90mpi_put_var_6D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_6D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 6

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_6D_text = &
          nfmpi_put_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1))
     else
       nf90mpi_put_var_6D_text = &
          nfmpi_put_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1))
     end if
   end function nf90mpi_put_var_6D_text

   function nf90mpi_put_var_7D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:,:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_7D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 7

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_7D_text = &
          nfmpi_put_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1,1))
     else
       nf90mpi_put_var_7D_text = &
          nfmpi_put_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1,1))
     end if
   end function nf90mpi_put_var_7D_text


   function nf90mpi_get_var_1D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_1D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 1

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_1D_text = &
          nfmpi_get_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1))
     else
       nf90mpi_get_var_1D_text = &
          nfmpi_get_vars_text(ncid, varid, localStart, localCount, localStride, values(1))
     end if
   end function nf90mpi_get_var_1D_text

   function nf90mpi_get_var_2D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_2D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 2

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_2D_text = &
          nfmpi_get_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1))
     else
       nf90mpi_get_var_2D_text = &
          nfmpi_get_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1))
     end if
   end function nf90mpi_get_var_2D_text

   function nf90mpi_get_var_3D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_3D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 3

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_3D_text = &
          nfmpi_get_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1))
     else
       nf90mpi_get_var_3D_text = &
          nfmpi_get_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1))
     end if
   end function nf90mpi_get_var_3D_text

   function nf90mpi_get_var_4D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_4D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 4

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_4D_text = &
          nfmpi_get_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1))
     else
       nf90mpi_get_var_4D_text = &
          nfmpi_get_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1))
     end if
   end function nf90mpi_get_var_4D_text

   function nf90mpi_get_var_5D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_5D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 5

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_5D_text = &
          nfmpi_get_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1))
     else
       nf90mpi_get_var_5D_text = &
          nfmpi_get_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1))
     end if
   end function nf90mpi_get_var_5D_text

   function nf90mpi_get_var_6D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_6D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 6

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_6D_text = &
          nfmpi_get_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1))
     else
       nf90mpi_get_var_6D_text = &
          nfmpi_get_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1))
     end if
   end function nf90mpi_get_var_6D_text

   function nf90mpi_get_var_7D_text(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:,:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_7D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 7

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_7D_text = &
          nfmpi_get_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1,1))
     else
       nf90mpi_get_var_7D_text = &
          nfmpi_get_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1,1))
     end if
   end function nf90mpi_get_var_7D_text


!
! Collective APIs
!

   function nf90mpi_put_var_1D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_1D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 1

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_1D_text_all = &
          nfmpi_put_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1))
     else
       nf90mpi_put_var_1D_text_all = &
          nfmpi_put_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1))
     end if
   end function nf90mpi_put_var_1D_text_all

   function nf90mpi_put_var_2D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_2D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 2

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_2D_text_all = &
          nfmpi_put_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1))
     else
       nf90mpi_put_var_2D_text_all = &
          nfmpi_put_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1))
     end if
   end function nf90mpi_put_var_2D_text_all

   function nf90mpi_put_var_3D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_3D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 3

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_3D_text_all = &
          nfmpi_put_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1))
     else
       nf90mpi_put_var_3D_text_all = &
          nfmpi_put_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1,1))
     end if
   end function nf90mpi_put_var_3D_text_all

   function nf90mpi_put_var_4D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_4D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 4

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_4D_text_all = &
          nfmpi_put_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1))
     else
       nf90mpi_put_var_4D_text_all = &
          nfmpi_put_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1,1,1))
     end if
   end function nf90mpi_put_var_4D_text_all

   function nf90mpi_put_var_5D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_5D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 5

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_5D_text_all = &
          nfmpi_put_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1))
     else
       nf90mpi_put_var_5D_text_all = &
          nfmpi_put_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1))
     end if
   end function nf90mpi_put_var_5D_text_all

   function nf90mpi_put_var_6D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_6D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 6

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_6D_text_all = &
          nfmpi_put_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1))
     else
       nf90mpi_put_var_6D_text_all = &
          nfmpi_put_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1))
     end if
   end function nf90mpi_put_var_6D_text_all

   function nf90mpi_put_var_7D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:,:,:),                       intent(in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_put_var_7D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 7

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_put_var_7D_text_all = &
          nfmpi_put_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1,1))
     else
       nf90mpi_put_var_7D_text_all = &
          nfmpi_put_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1,1))
     end if
   end function nf90mpi_put_var_7D_text_all


   function nf90mpi_get_var_1D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_1D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 1

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_1D_text_all = &
          nfmpi_get_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1))
     else
       nf90mpi_get_var_1D_text_all = &
          nfmpi_get_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1))
     end if
   end function nf90mpi_get_var_1D_text_all

   function nf90mpi_get_var_2D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_2D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 2

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_2D_text_all = &
          nfmpi_get_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1))
     else
       nf90mpi_get_var_2D_text_all = &
          nfmpi_get_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1))
     end if
   end function nf90mpi_get_var_2D_text_all

   function nf90mpi_get_var_3D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_3D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 3

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_3D_text_all = &
          nfmpi_get_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1))
     else
       nf90mpi_get_var_3D_text_all = &
          nfmpi_get_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1,1))
     end if
   end function nf90mpi_get_var_3D_text_all

   function nf90mpi_get_var_4D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_4D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 4

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_4D_text_all = &
          nfmpi_get_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1))
     else
       nf90mpi_get_var_4D_text_all = &
          nfmpi_get_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1,1,1))
     end if
   end function nf90mpi_get_var_4D_text_all

   function nf90mpi_get_var_5D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_5D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 5

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_5D_text_all = &
          nfmpi_get_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1))
     else
       nf90mpi_get_var_5D_text_all = &
          nfmpi_get_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1))
     end if
   end function nf90mpi_get_var_5D_text_all

   function nf90mpi_get_var_6D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_6D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 6

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_6D_text_all = &
          nfmpi_get_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1))
     else
       nf90mpi_get_var_6D_text_all = &
          nfmpi_get_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1))
     end if
   end function nf90mpi_get_var_6D_text_all

   function nf90mpi_get_var_7D_text_all(ncid, varid, values, start, count, stride, map)
     integer,                                                intent(in) :: ncid, varid
     character (len=*), dimension(:,:,:,:,:,:,:),                       intent(out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent(in) :: start, count, stride, map
 
     integer                                                            :: nf90mpi_get_var_7D_text_all
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)       :: localStart, localCount, localStride, localMap
     integer                                                            :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 7

     localStart (:         ) = 1
     localCount (:numDims+1) = (/ LEN(values(1,1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_get_var_7D_text_all = &
          nfmpi_get_varm_text_all(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1,1))
     else
       nf90mpi_get_var_7D_text_all = &
          nfmpi_get_vars_text_all(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1,1))
     end if
   end function nf90mpi_get_var_7D_text_all


!
! Nonblocking APIs
!


!
! iput APIs
!

   function nf90mpi_iput_var_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*),                                      intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map

     integer                                                             :: nf90mpi_iput_var_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride
 
     ! Set local arguments to default values
     localStart (:)  = 1
     localCount (1)  = LEN(values); localCount (2:) = 1
     localStride(:)  = 1
          
     if(present(start))  localStart (:size(start) ) = start(:)
     if(present(count))  localCount (:size(count) ) = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map)) then
       nf90mpi_iput_var_text = nfmpi_iput_varm_text(ncid, varid, localStart, localCount, localStride, map, values, req)
     else
       nf90mpi_iput_var_text = nfmpi_iput_vars_text(ncid, varid, localStart, localCount, localStride, values, req)
     end if
   end function nf90mpi_iput_var_text

   function nf90mpi_iget_var_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*),                                      intent( out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map

     integer                                                             :: nf90mpi_iget_var_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride
 
     ! Set local arguments to default values
     localStart (:)  = 1
     localCount (1)  = LEN(values); localCount (2:) = 1
     localStride(:)  = 1
          
     if(present(start))  localStart (:size(start) ) = start(:)
     if(present(count))  localCount (:size(count) ) = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map)) then
       nf90mpi_iget_var_text = nfmpi_iget_varm_text(ncid, varid, localStart, localCount, localStride, map, values, req)
     else
       nf90mpi_iget_var_text = nfmpi_iget_vars_text(ncid, varid, localStart, localCount, localStride, values, req)
     end if
   end function nf90mpi_iget_var_text


!
! bput APIs
!

   function nf90mpi_bput_var_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*),                                      intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map

     integer                                                             :: nf90mpi_bput_var_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride
 
     ! Set local arguments to default values
     localStart (:)  = 1
     localCount (1)  = LEN(values); localCount (2:) = 1
     localStride(:)  = 1
          
     if(present(start))  localStart (:size(start) ) = start(:)
     if(present(count))  localCount (:size(count) ) = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map)) then
       nf90mpi_bput_var_text = nfmpi_bput_varm_text(ncid, varid, localStart, localCount, localStride, map, values, req)
     else
       nf90mpi_bput_var_text = nfmpi_bput_vars_text(ncid, varid, localStart, localCount, localStride, values, req)
     end if
   end function nf90mpi_bput_var_text




   function nf90mpi_iput_var_1D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iput_var_1D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 1

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iput_var_1D_text = &
          nfmpi_iput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1), req)
     else
       nf90mpi_iput_var_1D_text = &
          nfmpi_iput_vars_text(ncid, varid, localStart, localCount, localStride, values(1), req)
     end if
   end function nf90mpi_iput_var_1D_text

   function nf90mpi_iput_var_2D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iput_var_2D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 2

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iput_var_2D_text = &
          nfmpi_iput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1), req)
     else
       nf90mpi_iput_var_2D_text = &
          nfmpi_iput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1), req)
     end if
   end function nf90mpi_iput_var_2D_text

   function nf90mpi_iput_var_3D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iput_var_3D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 3

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iput_var_3D_text = &
          nfmpi_iput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1), req)
     else
       nf90mpi_iput_var_3D_text = &
          nfmpi_iput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1), req)
     end if
   end function nf90mpi_iput_var_3D_text

   function nf90mpi_iput_var_4D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iput_var_4D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 4

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iput_var_4D_text = &
          nfmpi_iput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1), req)
     else
       nf90mpi_iput_var_4D_text = &
          nfmpi_iput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1), req)
     end if
   end function nf90mpi_iput_var_4D_text

   function nf90mpi_iput_var_5D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iput_var_5D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 5

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iput_var_5D_text = &
          nfmpi_iput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1), req)
     else
       nf90mpi_iput_var_5D_text = &
          nfmpi_iput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1), req)
     end if
   end function nf90mpi_iput_var_5D_text

   function nf90mpi_iput_var_6D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:,:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iput_var_6D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 6

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iput_var_6D_text = &
          nfmpi_iput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1), req)
     else
       nf90mpi_iput_var_6D_text = &
          nfmpi_iput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1), req)
     end if
   end function nf90mpi_iput_var_6D_text

   function nf90mpi_iput_var_7D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:,:,:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iput_var_7D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 7

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iput_var_7D_text = &
          nfmpi_iput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1,1), req)
     else
       nf90mpi_iput_var_7D_text = &
          nfmpi_iput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1,1), req)
     end if
   end function nf90mpi_iput_var_7D_text


   function nf90mpi_iget_var_1D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:),                       intent( out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iget_var_1D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 1

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iget_var_1D_text = &
          nfmpi_iget_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1), req)
     else
       nf90mpi_iget_var_1D_text = &
          nfmpi_iget_vars_text(ncid, varid, localStart, localCount, localStride, values(1), req)
     end if
   end function nf90mpi_iget_var_1D_text

   function nf90mpi_iget_var_2D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:),                       intent( out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iget_var_2D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 2

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iget_var_2D_text = &
          nfmpi_iget_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1), req)
     else
       nf90mpi_iget_var_2D_text = &
          nfmpi_iget_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1), req)
     end if
   end function nf90mpi_iget_var_2D_text

   function nf90mpi_iget_var_3D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:),                       intent( out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iget_var_3D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 3

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iget_var_3D_text = &
          nfmpi_iget_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1), req)
     else
       nf90mpi_iget_var_3D_text = &
          nfmpi_iget_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1), req)
     end if
   end function nf90mpi_iget_var_3D_text

   function nf90mpi_iget_var_4D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:),                       intent( out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iget_var_4D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 4

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iget_var_4D_text = &
          nfmpi_iget_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1), req)
     else
       nf90mpi_iget_var_4D_text = &
          nfmpi_iget_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1), req)
     end if
   end function nf90mpi_iget_var_4D_text

   function nf90mpi_iget_var_5D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:,:),                       intent( out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iget_var_5D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 5

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iget_var_5D_text = &
          nfmpi_iget_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1), req)
     else
       nf90mpi_iget_var_5D_text = &
          nfmpi_iget_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1), req)
     end if
   end function nf90mpi_iget_var_5D_text

   function nf90mpi_iget_var_6D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:,:,:),                       intent( out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iget_var_6D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 6

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iget_var_6D_text = &
          nfmpi_iget_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1), req)
     else
       nf90mpi_iget_var_6D_text = &
          nfmpi_iget_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1), req)
     end if
   end function nf90mpi_iget_var_6D_text

   function nf90mpi_iget_var_7D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:,:,:,:),                       intent( out) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_iget_var_7D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 7

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_iget_var_7D_text = &
          nfmpi_iget_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1,1), req)
     else
       nf90mpi_iget_var_7D_text = &
          nfmpi_iget_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1,1), req)
     end if
   end function nf90mpi_iget_var_7D_text


!
! bput APIs
!

   function nf90mpi_bput_var_1D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_bput_var_1D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 1

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_bput_var_1D_text = &
          nfmpi_bput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1), req)
     else
       nf90mpi_bput_var_1D_text = &
          nfmpi_bput_vars_text(ncid, varid, localStart, localCount, localStride, values(1), req)
     end if
   end function nf90mpi_bput_var_1D_text

   function nf90mpi_bput_var_2D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_bput_var_2D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 2

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_bput_var_2D_text = &
          nfmpi_bput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1), req)
     else
       nf90mpi_bput_var_2D_text = &
          nfmpi_bput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1), req)
     end if
   end function nf90mpi_bput_var_2D_text

   function nf90mpi_bput_var_3D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_bput_var_3D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 3

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_bput_var_3D_text = &
          nfmpi_bput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1), req)
     else
       nf90mpi_bput_var_3D_text = &
          nfmpi_bput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1), req)
     end if
   end function nf90mpi_bput_var_3D_text

   function nf90mpi_bput_var_4D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_bput_var_4D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 4

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_bput_var_4D_text = &
          nfmpi_bput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1), req)
     else
       nf90mpi_bput_var_4D_text = &
          nfmpi_bput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1), req)
     end if
   end function nf90mpi_bput_var_4D_text

   function nf90mpi_bput_var_5D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_bput_var_5D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 5

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_bput_var_5D_text = &
          nfmpi_bput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1), req)
     else
       nf90mpi_bput_var_5D_text = &
          nfmpi_bput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1), req)
     end if
   end function nf90mpi_bput_var_5D_text

   function nf90mpi_bput_var_6D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:,:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_bput_var_6D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 6

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_bput_var_6D_text = &
          nfmpi_bput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1), req)
     else
       nf90mpi_bput_var_6D_text = &
          nfmpi_bput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1), req)
     end if
   end function nf90mpi_bput_var_6D_text

   function nf90mpi_bput_var_7D_text(ncid, varid, values, req, start, count, stride, map)
     integer,                                                intent( in) :: ncid, varid
     integer,                                                intent(out) :: req
     character (len=*), dimension(:,:,:,:,:,:,:),                       intent( in) :: values
     integer (kind=MPI_OFFSET_KIND), dimension(:), optional, intent( in) :: start, count, stride, map
 
     integer                                                             :: nf90mpi_bput_var_7D_text
     integer (kind=MPI_OFFSET_KIND), dimension(nf90_max_var_dims)        :: localStart, localCount, localStride, localMap
     integer                                                             :: numDims, counter
 
     ! Set local arguments to default values
     numDims = 7

     localStart (:         ) = 1
     localCount ( :numDims+1) = (/ LEN(values(1,1,1,1,1,1,1)), shape(values) /)
     localCount (numDims+2:) = 0
     localStride(:         ) = 1
     ! localMap   (:numDims  ) = (/ 1, (product(localCount(:counter)), counter = 1, numDims - 1) /)
     localMap(1) = 1
     do counter = 1, numDims - 1
        localMap(counter+1) = localMap(counter) * localCount(counter)
     enddo
 
     if(present(start))  localStart (:size(start))  = start(:)
     if(present(count))  localCount (:size(count))  = count(:)
     if(present(stride)) localStride(:size(stride)) = stride(:)
     if(present(map))  then
       localMap   (:size(map))    = map(:)
       nf90mpi_bput_var_7D_text = &
          nfmpi_bput_varm_text(ncid, varid, localStart, localCount, localStride, localMap, values(1,1,1,1,1,1,1), req)
     else
       nf90mpi_bput_var_7D_text = &
          nfmpi_bput_vars_text(ncid, varid, localStart, localCount, localStride, values(1,1,1,1,1,1,1), req)
     end if
   end function nf90mpi_bput_var_7D_text


