/* Do not edit this file. It is produced from the corresponding .m4 source */
/*
 *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
 *  See COPYRIGHT notice in top-level directory.
 */
/* $Id: m_getput_varx.m4 2686 2016-12-07 20:02:41Z wkliao $ */

#if HAVE_CONFIG_H
# include <ncconfig.h>
#endif

#include <stdio.h>
#include <unistd.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#include <assert.h>

#include <mpi.h>

#include "nc.h"
#include "ncx.h"
#include "ncmpidtype.h"
#include "macro.h"


/* buffer layers:

        User Level              buf     (user defined buffer of MPI_Datatype)
        MPI Datatype Level      cbuf    (contiguous buffer of ptype)
        NetCDF XDR Level        xbuf    (XDR I/O buffer)
*/

static int
ncmpii_mgetput_varm(int                ncid,
                    int                num,
                    int                varids[],    /* [num] */
                    MPI_Offset* const  starts[],    /* [num] */
                    MPI_Offset* const  counts[],    /* [num] */
                    MPI_Offset* const  strides[],   /* [num] */
                    MPI_Offset* const  imaps[],     /* [num] */
                    void              *bufs[],      /* [num] */
                    MPI_Offset         bufcounts[], /* [num] */
                    MPI_Datatype       datatypes[], /* [num] */
                    int                rw_flag,     /* WRITE_REQ or READ_REQ */
                    int                io_method);  /* COLL_IO or INDEP_IO */

/*----< ncmpi_mput_var() >---------------------------------------------------*/
int
ncmpi_mput_var(int           ncid,
               int           num,
               int           varids[],    /* [num] */
               void         *bufs[],      /* [num] */
               MPI_Offset    bufcounts[], /* [num] */
               MPI_Datatype  datatypes[]) /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               WRITE_REQ, INDEP_IO);
}

/*----< ncmpi_mput_var_all() >-----------------------------------------------*/
int
ncmpi_mput_var_all(int           ncid,
                   int           num,
                   int           varids[],    /* [num] */
                   void         *bufs[],      /* [num] */
                   MPI_Offset    bufcounts[], /* [num] */
                   MPI_Datatype  datatypes[]) /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               WRITE_REQ, COLL_IO);
}


/*----< ncmpi_mput_var_text() >-----------------------------------------------*/
int
ncmpi_mput_var_text(int ncid,
                    int num,
                    int varids[],
                    char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_schar() >-----------------------------------------------*/
int
ncmpi_mput_var_schar(int ncid,
                    int num,
                    int varids[],
                    signed char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_uchar() >-----------------------------------------------*/
int
ncmpi_mput_var_uchar(int ncid,
                    int num,
                    int varids[],
                    unsigned char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_short() >-----------------------------------------------*/
int
ncmpi_mput_var_short(int ncid,
                    int num,
                    int varids[],
                    short *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_ushort() >-----------------------------------------------*/
int
ncmpi_mput_var_ushort(int ncid,
                    int num,
                    int varids[],
                    unsigned short *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_int() >-----------------------------------------------*/
int
ncmpi_mput_var_int(int ncid,
                    int num,
                    int varids[],
                    int *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_uint() >-----------------------------------------------*/
int
ncmpi_mput_var_uint(int ncid,
                    int num,
                    int varids[],
                    unsigned int *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_long() >-----------------------------------------------*/
int
ncmpi_mput_var_long(int ncid,
                    int num,
                    int varids[],
                    long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_float() >-----------------------------------------------*/
int
ncmpi_mput_var_float(int ncid,
                    int num,
                    int varids[],
                    float *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_double() >-----------------------------------------------*/
int
ncmpi_mput_var_double(int ncid,
                    int num,
                    int varids[],
                    double *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_longlong() >-----------------------------------------------*/
int
ncmpi_mput_var_longlong(int ncid,
                    int num,
                    int varids[],
                    long long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_ulonglong() >-----------------------------------------------*/
int
ncmpi_mput_var_ulonglong(int ncid,
                    int num,
                    int varids[],
                    unsigned long long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mput_var_text_all() >-----------------------------------------------*/
int
ncmpi_mput_var_text_all(int ncid,
                    int num,
                    int varids[],
                    char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_schar_all() >-----------------------------------------------*/
int
ncmpi_mput_var_schar_all(int ncid,
                    int num,
                    int varids[],
                    signed char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_uchar_all() >-----------------------------------------------*/
int
ncmpi_mput_var_uchar_all(int ncid,
                    int num,
                    int varids[],
                    unsigned char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_short_all() >-----------------------------------------------*/
int
ncmpi_mput_var_short_all(int ncid,
                    int num,
                    int varids[],
                    short *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_ushort_all() >-----------------------------------------------*/
int
ncmpi_mput_var_ushort_all(int ncid,
                    int num,
                    int varids[],
                    unsigned short *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_int_all() >-----------------------------------------------*/
int
ncmpi_mput_var_int_all(int ncid,
                    int num,
                    int varids[],
                    int *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_uint_all() >-----------------------------------------------*/
int
ncmpi_mput_var_uint_all(int ncid,
                    int num,
                    int varids[],
                    unsigned int *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_long_all() >-----------------------------------------------*/
int
ncmpi_mput_var_long_all(int ncid,
                    int num,
                    int varids[],
                    long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_float_all() >-----------------------------------------------*/
int
ncmpi_mput_var_float_all(int ncid,
                    int num,
                    int varids[],
                    float *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_double_all() >-----------------------------------------------*/
int
ncmpi_mput_var_double_all(int ncid,
                    int num,
                    int varids[],
                    double *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_longlong_all() >-----------------------------------------------*/
int
ncmpi_mput_var_longlong_all(int ncid,
                    int num,
                    int varids[],
                    long long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var_ulonglong_all() >-----------------------------------------------*/
int
ncmpi_mput_var_ulonglong_all(int ncid,
                    int num,
                    int varids[],
                    unsigned long long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mput_var1() >--------------------------------------------------*/
int
ncmpi_mput_var1(int                ncid,
                int                num,
                int                varids[],    /* [num] */
                MPI_Offset* const  starts[],    /* [num] */
                void              *bufs[],      /* [num] */
                MPI_Offset         bufcounts[], /* [num] */
                MPI_Datatype       datatypes[]) /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               WRITE_REQ, INDEP_IO);
}

/*----< ncmpi_mput_var1_all() >----------------------------------------------*/
int
ncmpi_mput_var1_all(int                ncid,
                    int                num,
                    int                varids[],    /* [num] */
                    MPI_Offset* const  starts[],    /* [num] */
                    void              *bufs[],      /* [num] */
                    MPI_Offset         bufcounts[], /* [num] */
                    MPI_Datatype       datatypes[]) /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               WRITE_REQ, COLL_IO);
}


/*----< ncmpi_mput_var1_text() >----------------------------------------------*/
int
ncmpi_mput_var1_text(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_schar() >----------------------------------------------*/
int
ncmpi_mput_var1_schar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_uchar() >----------------------------------------------*/
int
ncmpi_mput_var1_uchar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_short() >----------------------------------------------*/
int
ncmpi_mput_var1_short(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_ushort() >----------------------------------------------*/
int
ncmpi_mput_var1_ushort(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_int() >----------------------------------------------*/
int
ncmpi_mput_var1_int(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_uint() >----------------------------------------------*/
int
ncmpi_mput_var1_uint(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_long() >----------------------------------------------*/
int
ncmpi_mput_var1_long(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_float() >----------------------------------------------*/
int
ncmpi_mput_var1_float(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_double() >----------------------------------------------*/
int
ncmpi_mput_var1_double(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_longlong() >----------------------------------------------*/
int
ncmpi_mput_var1_longlong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_ulonglong() >----------------------------------------------*/
int
ncmpi_mput_var1_ulonglong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mput_var1_text_all() >----------------------------------------------*/
int
ncmpi_mput_var1_text_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_schar_all() >----------------------------------------------*/
int
ncmpi_mput_var1_schar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_uchar_all() >----------------------------------------------*/
int
ncmpi_mput_var1_uchar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_short_all() >----------------------------------------------*/
int
ncmpi_mput_var1_short_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_ushort_all() >----------------------------------------------*/
int
ncmpi_mput_var1_ushort_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_int_all() >----------------------------------------------*/
int
ncmpi_mput_var1_int_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_uint_all() >----------------------------------------------*/
int
ncmpi_mput_var1_uint_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_long_all() >----------------------------------------------*/
int
ncmpi_mput_var1_long_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_float_all() >----------------------------------------------*/
int
ncmpi_mput_var1_float_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_double_all() >----------------------------------------------*/
int
ncmpi_mput_var1_double_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_longlong_all() >----------------------------------------------*/
int
ncmpi_mput_var1_longlong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_var1_ulonglong_all() >----------------------------------------------*/
int
ncmpi_mput_var1_ulonglong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mput_vara() >--------------------------------------------------*/
int
ncmpi_mput_vara(int                ncid,
                int                num,
                int                varids[],    /* [num] */
                MPI_Offset* const  starts[],    /* [num] */
                MPI_Offset* const  counts[],    /* [num] */
                void              *bufs[],      /* [num] */
                MPI_Offset         bufcounts[], /* [num] */
                MPI_Datatype       datatypes[]) /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               WRITE_REQ, INDEP_IO);
}

/*----< ncmpi_mput_vara_all() >----------------------------------------------*/
int
ncmpi_mput_vara_all(int                ncid,
                    int                num,
                    int                varids[],    /* [num] */
                    MPI_Offset* const  starts[],    /* [num] */
                    MPI_Offset* const  counts[],    /* [num] */
                    void              *bufs[],      /* [num] */
                    MPI_Offset         bufcounts[], /* [num] */
                    MPI_Datatype       datatypes[]) /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               WRITE_REQ, COLL_IO);
}


/*----< ncmpi_mput_vara_text() >----------------------------------------------*/
int
ncmpi_mput_vara_text(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_schar() >----------------------------------------------*/
int
ncmpi_mput_vara_schar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_uchar() >----------------------------------------------*/
int
ncmpi_mput_vara_uchar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_short() >----------------------------------------------*/
int
ncmpi_mput_vara_short(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_ushort() >----------------------------------------------*/
int
ncmpi_mput_vara_ushort(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_int() >----------------------------------------------*/
int
ncmpi_mput_vara_int(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_uint() >----------------------------------------------*/
int
ncmpi_mput_vara_uint(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_long() >----------------------------------------------*/
int
ncmpi_mput_vara_long(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_float() >----------------------------------------------*/
int
ncmpi_mput_vara_float(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_double() >----------------------------------------------*/
int
ncmpi_mput_vara_double(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_longlong() >----------------------------------------------*/
int
ncmpi_mput_vara_longlong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_ulonglong() >----------------------------------------------*/
int
ncmpi_mput_vara_ulonglong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mput_vara_text_all() >----------------------------------------------*/
int
ncmpi_mput_vara_text_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_schar_all() >----------------------------------------------*/
int
ncmpi_mput_vara_schar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_uchar_all() >----------------------------------------------*/
int
ncmpi_mput_vara_uchar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_short_all() >----------------------------------------------*/
int
ncmpi_mput_vara_short_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_ushort_all() >----------------------------------------------*/
int
ncmpi_mput_vara_ushort_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_int_all() >----------------------------------------------*/
int
ncmpi_mput_vara_int_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_uint_all() >----------------------------------------------*/
int
ncmpi_mput_vara_uint_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_long_all() >----------------------------------------------*/
int
ncmpi_mput_vara_long_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_float_all() >----------------------------------------------*/
int
ncmpi_mput_vara_float_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_double_all() >----------------------------------------------*/
int
ncmpi_mput_vara_double_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_longlong_all() >----------------------------------------------*/
int
ncmpi_mput_vara_longlong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vara_ulonglong_all() >----------------------------------------------*/
int
ncmpi_mput_vara_ulonglong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mput_vars() >--------------------------------------------------*/
int
ncmpi_mput_vars(int                ncid,
                int                num,
                int                varids[],    /* [num] */
                MPI_Offset* const  starts[],    /* [num] */
                MPI_Offset* const  counts[],    /* [num] */
                MPI_Offset* const  strides[],   /* [num] */
                void              *bufs[],      /* [num] */
                MPI_Offset         bufcounts[], /* [num] */
                MPI_Datatype       datatypes[]) /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                               NULL, bufs, bufcounts, datatypes,
                               WRITE_REQ, INDEP_IO);
}

/*----< ncmpi_mput_vars_all() >----------------------------------------------*/
int
ncmpi_mput_vars_all(int                ncid,
                    int                num,
                    int                varids[],    /* [num] */
                    MPI_Offset* const  starts[],    /* [num] */
                    MPI_Offset* const  counts[],    /* [num] */
                    MPI_Offset* const  strides[],   /* [num] */
                    void              *bufs[],      /* [num] */
                    MPI_Offset         bufcounts[], /* [num] */
                    MPI_Datatype       datatypes[]) /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                               NULL, bufs, bufcounts, datatypes,
                               WRITE_REQ, COLL_IO);
}


/*----< ncmpi_mput_vars_text() >----------------------------------------------*/
int
ncmpi_mput_vars_text(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_schar() >----------------------------------------------*/
int
ncmpi_mput_vars_schar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_uchar() >----------------------------------------------*/
int
ncmpi_mput_vars_uchar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_short() >----------------------------------------------*/
int
ncmpi_mput_vars_short(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_ushort() >----------------------------------------------*/
int
ncmpi_mput_vars_ushort(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_int() >----------------------------------------------*/
int
ncmpi_mput_vars_int(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_uint() >----------------------------------------------*/
int
ncmpi_mput_vars_uint(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_long() >----------------------------------------------*/
int
ncmpi_mput_vars_long(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_float() >----------------------------------------------*/
int
ncmpi_mput_vars_float(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_double() >----------------------------------------------*/
int
ncmpi_mput_vars_double(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_longlong() >----------------------------------------------*/
int
ncmpi_mput_vars_longlong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_ulonglong() >----------------------------------------------*/
int
ncmpi_mput_vars_ulonglong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mput_vars_text_all() >----------------------------------------------*/
int
ncmpi_mput_vars_text_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_schar_all() >----------------------------------------------*/
int
ncmpi_mput_vars_schar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_uchar_all() >----------------------------------------------*/
int
ncmpi_mput_vars_uchar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_short_all() >----------------------------------------------*/
int
ncmpi_mput_vars_short_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_ushort_all() >----------------------------------------------*/
int
ncmpi_mput_vars_ushort_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_int_all() >----------------------------------------------*/
int
ncmpi_mput_vars_int_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_uint_all() >----------------------------------------------*/
int
ncmpi_mput_vars_uint_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_long_all() >----------------------------------------------*/
int
ncmpi_mput_vars_long_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_float_all() >----------------------------------------------*/
int
ncmpi_mput_vars_float_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_double_all() >----------------------------------------------*/
int
ncmpi_mput_vars_double_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_longlong_all() >----------------------------------------------*/
int
ncmpi_mput_vars_longlong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_vars_ulonglong_all() >----------------------------------------------*/
int
ncmpi_mput_vars_ulonglong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mput_varm() >--------------------------------------------------*/
int
ncmpi_mput_varm(int                ncid,
                int                num,
                int                varids[],    /* [num] */
                MPI_Offset* const  starts[],    /* [num] */
                MPI_Offset* const  counts[],    /* [num] */
                MPI_Offset* const  strides[],   /* [num] */
                MPI_Offset* const  imaps[],     /* [num] */
                void              *bufs[],      /* [num] */
                MPI_Offset         bufcounts[], /* [num] */
                MPI_Datatype       datatypes[]) /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                               imaps, bufs, bufcounts, datatypes,
                               WRITE_REQ, INDEP_IO);
}

/*----< ncmpi_mput_varm_all() >----------------------------------------------*/
int
ncmpi_mput_varm_all(int                ncid,
                    int                num,
                    int                varids[],    /* [num] */
                    MPI_Offset* const  starts[],    /* [num] */
                    MPI_Offset* const  counts[],    /* [num] */
                    MPI_Offset* const  strides[],   /* [num] */
                    MPI_Offset* const  imaps[],     /* [num] */
                    void              *bufs[],      /* [num] */
                    MPI_Offset         bufcounts[], /* [num] */
                    MPI_Datatype       datatypes[]) /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                               imaps, bufs, bufcounts, datatypes,
                               WRITE_REQ, COLL_IO);
}


/*----< ncmpi_mput_varm_text() >----------------------------------------------*/
int
ncmpi_mput_varm_text(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_schar() >----------------------------------------------*/
int
ncmpi_mput_varm_schar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_uchar() >----------------------------------------------*/
int
ncmpi_mput_varm_uchar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_short() >----------------------------------------------*/
int
ncmpi_mput_varm_short(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_ushort() >----------------------------------------------*/
int
ncmpi_mput_varm_ushort(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_int() >----------------------------------------------*/
int
ncmpi_mput_varm_int(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_uint() >----------------------------------------------*/
int
ncmpi_mput_varm_uint(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_long() >----------------------------------------------*/
int
ncmpi_mput_varm_long(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_float() >----------------------------------------------*/
int
ncmpi_mput_varm_float(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_double() >----------------------------------------------*/
int
ncmpi_mput_varm_double(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_longlong() >----------------------------------------------*/
int
ncmpi_mput_varm_longlong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_ulonglong() >----------------------------------------------*/
int
ncmpi_mput_varm_ulonglong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mput_varm_text_all() >----------------------------------------------*/
int
ncmpi_mput_varm_text_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_schar_all() >----------------------------------------------*/
int
ncmpi_mput_varm_schar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_uchar_all() >----------------------------------------------*/
int
ncmpi_mput_varm_uchar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_short_all() >----------------------------------------------*/
int
ncmpi_mput_varm_short_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_ushort_all() >----------------------------------------------*/
int
ncmpi_mput_varm_ushort_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_int_all() >----------------------------------------------*/
int
ncmpi_mput_varm_int_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_uint_all() >----------------------------------------------*/
int
ncmpi_mput_varm_uint_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_long_all() >----------------------------------------------*/
int
ncmpi_mput_varm_long_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_float_all() >----------------------------------------------*/
int
ncmpi_mput_varm_float_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_double_all() >----------------------------------------------*/
int
ncmpi_mput_varm_double_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_longlong_all() >----------------------------------------------*/
int
ncmpi_mput_varm_longlong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mput_varm_ulonglong_all() >----------------------------------------------*/
int
ncmpi_mput_varm_ulonglong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              WRITE_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mget_var() >---------------------------------------------------*/
int
ncmpi_mget_var(int           ncid,
               int           num,
               int           varids[],     /* [num] */
               void         *bufs[],       /* [num] */
               MPI_Offset    bufcounts[],  /* [num] */
               MPI_Datatype  datatypes[])  /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               READ_REQ, INDEP_IO);
}

/*----< ncmpi_mget_var_all() >-----------------------------------------------*/
int
ncmpi_mget_var_all(int           ncid,
                   int           num,
                   int           varids[],     /* [num] */
                   void         *bufs[],       /* [num] */
                   MPI_Offset    bufcounts[],  /* [num] */
                   MPI_Datatype  datatypes[])  /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               READ_REQ, COLL_IO);
}


/*----< ncmpi_mget_var_text() >-----------------------------------------------*/
int
ncmpi_mget_var_text(int ncid,
                    int num,
                    int varids[],
                    char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_schar() >-----------------------------------------------*/
int
ncmpi_mget_var_schar(int ncid,
                    int num,
                    int varids[],
                    signed char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_uchar() >-----------------------------------------------*/
int
ncmpi_mget_var_uchar(int ncid,
                    int num,
                    int varids[],
                    unsigned char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_short() >-----------------------------------------------*/
int
ncmpi_mget_var_short(int ncid,
                    int num,
                    int varids[],
                    short *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_ushort() >-----------------------------------------------*/
int
ncmpi_mget_var_ushort(int ncid,
                    int num,
                    int varids[],
                    unsigned short *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_int() >-----------------------------------------------*/
int
ncmpi_mget_var_int(int ncid,
                    int num,
                    int varids[],
                    int *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_uint() >-----------------------------------------------*/
int
ncmpi_mget_var_uint(int ncid,
                    int num,
                    int varids[],
                    unsigned int *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_long() >-----------------------------------------------*/
int
ncmpi_mget_var_long(int ncid,
                    int num,
                    int varids[],
                    long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_float() >-----------------------------------------------*/
int
ncmpi_mget_var_float(int ncid,
                    int num,
                    int varids[],
                    float *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_double() >-----------------------------------------------*/
int
ncmpi_mget_var_double(int ncid,
                    int num,
                    int varids[],
                    double *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_longlong() >-----------------------------------------------*/
int
ncmpi_mget_var_longlong(int ncid,
                    int num,
                    int varids[],
                    long long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_ulonglong() >-----------------------------------------------*/
int
ncmpi_mget_var_ulonglong(int ncid,
                    int num,
                    int varids[],
                    unsigned long long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mget_var_text_all() >-----------------------------------------------*/
int
ncmpi_mget_var_text_all(int ncid,
                    int num,
                    int varids[],
                    char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_schar_all() >-----------------------------------------------*/
int
ncmpi_mget_var_schar_all(int ncid,
                    int num,
                    int varids[],
                    signed char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_uchar_all() >-----------------------------------------------*/
int
ncmpi_mget_var_uchar_all(int ncid,
                    int num,
                    int varids[],
                    unsigned char *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_short_all() >-----------------------------------------------*/
int
ncmpi_mget_var_short_all(int ncid,
                    int num,
                    int varids[],
                    short *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_ushort_all() >-----------------------------------------------*/
int
ncmpi_mget_var_ushort_all(int ncid,
                    int num,
                    int varids[],
                    unsigned short *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_int_all() >-----------------------------------------------*/
int
ncmpi_mget_var_int_all(int ncid,
                    int num,
                    int varids[],
                    int *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_uint_all() >-----------------------------------------------*/
int
ncmpi_mget_var_uint_all(int ncid,
                    int num,
                    int varids[],
                    unsigned int *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_long_all() >-----------------------------------------------*/
int
ncmpi_mget_var_long_all(int ncid,
                    int num,
                    int varids[],
                    long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_float_all() >-----------------------------------------------*/
int
ncmpi_mget_var_float_all(int ncid,
                    int num,
                    int varids[],
                    float *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_double_all() >-----------------------------------------------*/
int
ncmpi_mget_var_double_all(int ncid,
                    int num,
                    int varids[],
                    double *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_longlong_all() >-----------------------------------------------*/
int
ncmpi_mget_var_longlong_all(int ncid,
                    int num,
                    int varids[],
                    long long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var_ulonglong_all() >-----------------------------------------------*/
int
ncmpi_mget_var_ulonglong_all(int ncid,
                    int num,
                    int varids[],
                    unsigned long long *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, NULL, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mget_var1() >--------------------------------------------------*/
int
ncmpi_mget_var1(int                ncid,
                int                num,
                int                varids[],     /* [num] */
                MPI_Offset* const  starts[],     /* [num] */
                void              *bufs[],       /* [num] */
                MPI_Offset         bufcounts[],  /* [num] */
                MPI_Datatype       datatypes[])  /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               READ_REQ, INDEP_IO);
}

/*----< ncmpi_mget_var1_all() >----------------------------------------------*/
int
ncmpi_mget_var1_all(int                ncid,
                    int                num,
                    int                varids[],     /* [num] */
                    MPI_Offset* const  starts[],     /* [num] */
                    void              *bufs[],       /* [num] */
                    MPI_Offset         bufcounts[],  /* [num] */
                    MPI_Datatype       datatypes[])  /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               READ_REQ, COLL_IO);
}


/*----< ncmpi_mget_var1_text() >----------------------------------------------*/
int
ncmpi_mget_var1_text(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_schar() >----------------------------------------------*/
int
ncmpi_mget_var1_schar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_uchar() >----------------------------------------------*/
int
ncmpi_mget_var1_uchar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_short() >----------------------------------------------*/
int
ncmpi_mget_var1_short(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_ushort() >----------------------------------------------*/
int
ncmpi_mget_var1_ushort(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_int() >----------------------------------------------*/
int
ncmpi_mget_var1_int(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_uint() >----------------------------------------------*/
int
ncmpi_mget_var1_uint(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_long() >----------------------------------------------*/
int
ncmpi_mget_var1_long(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_float() >----------------------------------------------*/
int
ncmpi_mget_var1_float(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_double() >----------------------------------------------*/
int
ncmpi_mget_var1_double(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_longlong() >----------------------------------------------*/
int
ncmpi_mget_var1_longlong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_ulonglong() >----------------------------------------------*/
int
ncmpi_mget_var1_ulonglong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mget_var1_text_all() >----------------------------------------------*/
int
ncmpi_mget_var1_text_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_schar_all() >----------------------------------------------*/
int
ncmpi_mget_var1_schar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_uchar_all() >----------------------------------------------*/
int
ncmpi_mget_var1_uchar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_short_all() >----------------------------------------------*/
int
ncmpi_mget_var1_short_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_ushort_all() >----------------------------------------------*/
int
ncmpi_mget_var1_ushort_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_int_all() >----------------------------------------------*/
int
ncmpi_mget_var1_int_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_uint_all() >----------------------------------------------*/
int
ncmpi_mget_var1_uint_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_long_all() >----------------------------------------------*/
int
ncmpi_mget_var1_long_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_float_all() >----------------------------------------------*/
int
ncmpi_mget_var1_float_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_double_all() >----------------------------------------------*/
int
ncmpi_mget_var1_double_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_longlong_all() >----------------------------------------------*/
int
ncmpi_mget_var1_longlong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_var1_ulonglong_all() >----------------------------------------------*/
int
ncmpi_mget_var1_ulonglong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, NULL, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mget_vara() >--------------------------------------------------*/
int
ncmpi_mget_vara(int                ncid,
                int                num,
                int                varids[],     /* [num] */
                MPI_Offset* const  starts[],     /* [num] */
                MPI_Offset* const  counts[],     /* [num] */
                void              *bufs[],       /* [num] */
                MPI_Offset         bufcounts[],  /* [num] */
                MPI_Datatype       datatypes[])  /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               READ_REQ, INDEP_IO);
}

/*----< ncmpi_mget_vara_all() >----------------------------------------------*/
int
ncmpi_mget_vara_all(int                ncid,
                    int                num,
                    int                varids[],     /* [num] */
                    MPI_Offset* const  starts[],     /* [num] */
                    MPI_Offset* const  counts[],     /* [num] */
                    void              *bufs[],       /* [num] */
                    MPI_Offset         bufcounts[],  /* [num] */
                    MPI_Datatype       datatypes[])  /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                               NULL, bufs, bufcounts, datatypes,
                               READ_REQ, COLL_IO);
}


/*----< ncmpi_mget_vara_text() >----------------------------------------------*/
int
ncmpi_mget_vara_text(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_schar() >----------------------------------------------*/
int
ncmpi_mget_vara_schar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_uchar() >----------------------------------------------*/
int
ncmpi_mget_vara_uchar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_short() >----------------------------------------------*/
int
ncmpi_mget_vara_short(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_ushort() >----------------------------------------------*/
int
ncmpi_mget_vara_ushort(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_int() >----------------------------------------------*/
int
ncmpi_mget_vara_int(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_uint() >----------------------------------------------*/
int
ncmpi_mget_vara_uint(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_long() >----------------------------------------------*/
int
ncmpi_mget_vara_long(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_float() >----------------------------------------------*/
int
ncmpi_mget_vara_float(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_double() >----------------------------------------------*/
int
ncmpi_mget_vara_double(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_longlong() >----------------------------------------------*/
int
ncmpi_mget_vara_longlong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_ulonglong() >----------------------------------------------*/
int
ncmpi_mget_vara_ulonglong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mget_vara_text_all() >----------------------------------------------*/
int
ncmpi_mget_vara_text_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_schar_all() >----------------------------------------------*/
int
ncmpi_mget_vara_schar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_uchar_all() >----------------------------------------------*/
int
ncmpi_mget_vara_uchar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_short_all() >----------------------------------------------*/
int
ncmpi_mget_vara_short_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_ushort_all() >----------------------------------------------*/
int
ncmpi_mget_vara_ushort_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_int_all() >----------------------------------------------*/
int
ncmpi_mget_vara_int_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_uint_all() >----------------------------------------------*/
int
ncmpi_mget_vara_uint_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_long_all() >----------------------------------------------*/
int
ncmpi_mget_vara_long_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_float_all() >----------------------------------------------*/
int
ncmpi_mget_vara_float_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_double_all() >----------------------------------------------*/
int
ncmpi_mget_vara_double_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_longlong_all() >----------------------------------------------*/
int
ncmpi_mget_vara_longlong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vara_ulonglong_all() >----------------------------------------------*/
int
ncmpi_mget_vara_ulonglong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, NULL,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mget_vars() >--------------------------------------------------*/
int
ncmpi_mget_vars(int                ncid,
                int                num,
                int                varids[],     /* [num] */
                MPI_Offset* const  starts[],     /* [num] */
                MPI_Offset* const  counts[],     /* [num] */
                MPI_Offset* const  strides[],    /* [num] */
                void              *bufs[],       /* [num] */
                MPI_Offset         bufcounts[],  /* [num] */
                MPI_Datatype       datatypes[])  /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                               NULL, bufs, bufcounts, datatypes,
                               READ_REQ, INDEP_IO);
}

/*----< ncmpi_mget_vars_all() >----------------------------------------------*/
int
ncmpi_mget_vars_all(int                ncid,
                    int                num,
                    int                varids[],     /* [num] */
                    MPI_Offset* const  starts[],     /* [num] */
                    MPI_Offset* const  counts[],     /* [num] */
                    MPI_Offset* const  strides[],    /* [num] */
                    void              *bufs[],       /* [num] */
                    MPI_Offset         bufcounts[],  /* [num] */
                    MPI_Datatype       datatypes[])  /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                               NULL, bufs, bufcounts, datatypes,
                               READ_REQ, COLL_IO);
}


/*----< ncmpi_mget_vars_text() >----------------------------------------------*/
int
ncmpi_mget_vars_text(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_schar() >----------------------------------------------*/
int
ncmpi_mget_vars_schar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_uchar() >----------------------------------------------*/
int
ncmpi_mget_vars_uchar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_short() >----------------------------------------------*/
int
ncmpi_mget_vars_short(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_ushort() >----------------------------------------------*/
int
ncmpi_mget_vars_ushort(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_int() >----------------------------------------------*/
int
ncmpi_mget_vars_int(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_uint() >----------------------------------------------*/
int
ncmpi_mget_vars_uint(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_long() >----------------------------------------------*/
int
ncmpi_mget_vars_long(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_float() >----------------------------------------------*/
int
ncmpi_mget_vars_float(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_double() >----------------------------------------------*/
int
ncmpi_mget_vars_double(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_longlong() >----------------------------------------------*/
int
ncmpi_mget_vars_longlong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_ulonglong() >----------------------------------------------*/
int
ncmpi_mget_vars_ulonglong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mget_vars_text_all() >----------------------------------------------*/
int
ncmpi_mget_vars_text_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_schar_all() >----------------------------------------------*/
int
ncmpi_mget_vars_schar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_uchar_all() >----------------------------------------------*/
int
ncmpi_mget_vars_uchar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_short_all() >----------------------------------------------*/
int
ncmpi_mget_vars_short_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_ushort_all() >----------------------------------------------*/
int
ncmpi_mget_vars_ushort_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_int_all() >----------------------------------------------*/
int
ncmpi_mget_vars_int_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_uint_all() >----------------------------------------------*/
int
ncmpi_mget_vars_uint_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_long_all() >----------------------------------------------*/
int
ncmpi_mget_vars_long_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_float_all() >----------------------------------------------*/
int
ncmpi_mget_vars_float_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_double_all() >----------------------------------------------*/
int
ncmpi_mget_vars_double_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_longlong_all() >----------------------------------------------*/
int
ncmpi_mget_vars_longlong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_vars_ulonglong_all() >----------------------------------------------*/
int
ncmpi_mget_vars_ulonglong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              NULL, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mget_varm() >--------------------------------------------------*/
int
ncmpi_mget_varm(int                ncid,
                int                num,
                int                varids[],     /* [num] */
                MPI_Offset* const  starts[],     /* [num] */
                MPI_Offset* const  counts[],     /* [num] */
                MPI_Offset* const  strides[],    /* [num] */
                MPI_Offset* const  imaps[],      /* [num] */
                void              *bufs[],       /* [num] */
                MPI_Offset         bufcounts[],  /* [num] */
                MPI_Datatype       datatypes[])  /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                               imaps, bufs, bufcounts, datatypes,
                               READ_REQ, INDEP_IO);
}

/*----< ncmpi_mget_varm_all() >----------------------------------------------*/
int
ncmpi_mget_varm_all(int                ncid,
                    int                num,
                    int                varids[],     /* [num] */
                    MPI_Offset* const  starts[],     /* [num] */
                    MPI_Offset* const  counts[],     /* [num] */
                    MPI_Offset* const  strides[],    /* [num] */
                    MPI_Offset* const  imaps[],      /* [num] */
                    void              *bufs[],       /* [num] */
                    MPI_Offset         bufcounts[],  /* [num] */
                    MPI_Datatype       datatypes[])  /* [num] */
{
    return ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                               imaps, bufs, bufcounts, datatypes,
                               READ_REQ, COLL_IO);
}


/*----< ncmpi_mget_varm_text() >----------------------------------------------*/
int
ncmpi_mget_varm_text(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_schar() >----------------------------------------------*/
int
ncmpi_mget_varm_schar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_uchar() >----------------------------------------------*/
int
ncmpi_mget_varm_uchar(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_short() >----------------------------------------------*/
int
ncmpi_mget_varm_short(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_ushort() >----------------------------------------------*/
int
ncmpi_mget_varm_ushort(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_int() >----------------------------------------------*/
int
ncmpi_mget_varm_int(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_uint() >----------------------------------------------*/
int
ncmpi_mget_varm_uint(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_long() >----------------------------------------------*/
int
ncmpi_mget_varm_long(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_float() >----------------------------------------------*/
int
ncmpi_mget_varm_float(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_double() >----------------------------------------------*/
int
ncmpi_mget_varm_double(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_longlong() >----------------------------------------------*/
int
ncmpi_mget_varm_longlong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_ulonglong() >----------------------------------------------*/
int
ncmpi_mget_varm_ulonglong(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, INDEP_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpi_mget_varm_text_all() >----------------------------------------------*/
int
ncmpi_mget_varm_text_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_schar_all() >----------------------------------------------*/
int
ncmpi_mget_varm_schar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     signed char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_uchar_all() >----------------------------------------------*/
int
ncmpi_mget_varm_uchar_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned char                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_CHAR;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_short_all() >----------------------------------------------*/
int
ncmpi_mget_varm_short_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_ushort_all() >----------------------------------------------*/
int
ncmpi_mget_varm_ushort_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned short                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_SHORT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_int_all() >----------------------------------------------*/
int
ncmpi_mget_varm_int_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_uint_all() >----------------------------------------------*/
int
ncmpi_mget_varm_uint_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned int                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_long_all() >----------------------------------------------*/
int
ncmpi_mget_varm_long_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_float_all() >----------------------------------------------*/
int
ncmpi_mget_varm_float_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     float                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_FLOAT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_double_all() >----------------------------------------------*/
int
ncmpi_mget_varm_double_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     double                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_DOUBLE;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_longlong_all() >----------------------------------------------*/
int
ncmpi_mget_varm_longlong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_LONG_LONG_INT;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}

/*----< ncmpi_mget_varm_ulonglong_all() >----------------------------------------------*/
int
ncmpi_mget_varm_ulonglong_all(int                ncid,
                     int                num,
                     int                varids[],
                     MPI_Offset* const  starts[],    /* [num] */
                     MPI_Offset* const  counts[],    /* [num] */
                     MPI_Offset* const  strides[],   /* [num] */
                     MPI_Offset* const  imaps[],     /* [num] */
                     unsigned long long                *bufs[])
{
    int i, err;
    MPI_Datatype *datatypes;

    datatypes = (MPI_Datatype*) NCI_Malloc((size_t)num * sizeof(MPI_Datatype));
    for (i=0; i<num; i++)
        datatypes[i] = MPI_UNSIGNED_LONG_LONG;

    err = ncmpii_mgetput_varm(ncid, num, varids, starts, counts, strides,
                              imaps, (void**)bufs, NULL, datatypes,
                              READ_REQ, COLL_IO);
    NCI_Free(datatypes);
    return err;
}


/*----< ncmpii_mgetput_varm() >-----------------------------------------------*/
int
ncmpii_mgetput_varm(int                ncid,
                    int                num,
                    int                varids[],    /* [num] */
                    MPI_Offset* const  starts[],    /* [num] */
                    MPI_Offset* const  counts[],    /* [num] */
                    MPI_Offset* const  strides[],   /* [num] */
                    MPI_Offset* const  imaps[],     /* [num] */
                    void              *bufs[],      /* [num] */
                    MPI_Offset         bufcounts[], /* [num] */
                    MPI_Datatype       datatypes[], /* [num] */
                    int                rw_flag,     /* WRITE_REQ or READ_REQ */
                    int                io_method)   /* COLL_IO or INDEP_IO */
{
    int i, j, err, status=NC_NOERR, *req_ids=NULL, *statuses=NULL;
    NC *ncp=NULL;

    /* check if ncid is valid */
    status = ncmpii_NC_check_id(ncid, &ncp);
    if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
    /* if ncid is invalid, we must return the error now. However, the program
     * might hang when only a subset or processes have this error.
     */

    /* check if it is in define mode. This must be called in data mode */
    if (NC_indef(ncp)) {
        DEBUG_ASSIGN_ERROR(status, NC_EINDEFINE)
        goto err_check;
    }

    /* check file write permission if this is write request */
    if (rw_flag == WRITE_REQ && NC_readonly(ncp)) {
        DEBUG_ASSIGN_ERROR(status, NC_EPERM)
        goto err_check;
    }

err_check:
    /* at this point, if status is not NC_NOERR, it is a fatal error */
    if (ncp->safe_mode == 1 && io_method == COLL_IO) {
        int mpireturn, min_st;
        TRACE_COMM(MPI_Allreduce)(&status, &min_st, 1, MPI_INT, MPI_MIN,
                                  ncp->nciop->comm);
        if (mpireturn != MPI_SUCCESS)
            return ncmpii_handle_error(mpireturn, "MPI_Allreduce"); 
        if (min_st != NC_NOERR) return status;
    }

    if (io_method == INDEP_IO && status != NC_NOERR)
        return status;

    if (status != NC_NOERR)
        /* this can only be reached for COLL_IO and safe_mode == 0, set num=0
         * just so this process can participate the collective calls in
         * wait_all */
        num = 0;

    if (num > 0) {
        req_ids  = (int*) NCI_Malloc((size_t)(2 * num * SIZEOF_INT));
        statuses = req_ids + num;
    }

    /* for each request call ncmpi_igetput_varm() */
    for (i=0; i<num; i++) {
        NC_var *varp;
        MPI_Offset *start, *count, buflen=0;

        req_ids[i] = NC_REQ_NULL;

        err = ncmpii_NC_lookupvar(ncp, varids[i], &varp);
        if (err != NC_NOERR) {
            if (status == NC_NOERR) status = err;
            continue; /* invalid varid, skip this request */
        }

        if (bufcounts != NULL) buflen = bufcounts[i];

        if (starts == NULL) {         /* var */
            GET_FULL_DIMENSIONS(start, count)
            /* when bufcounts == NULL, it means the same as counts[] */
            if (bufcounts == NULL) {
                for (buflen=1, j=0; j<varp->ndims; j++)
                    buflen *= count[j];
            }
            err = ncmpii_igetput_varm(ncp, varp, start, count, NULL, NULL,
                                      bufs[i], buflen, datatypes[i],
                                      &req_ids[i], rw_flag, 0, 0);
            if (status == NC_NOERR) status = err;
            NCI_Free(start);
        } else if (counts == NULL) {  /* var1 */
            /* check whether starts[i] is valid */
            err = NC_start_count_stride_ck(ncp, varp, starts[i], NULL, NULL, rw_flag);
            if (status == NC_NOERR) status = err;
            if (err != NC_NOERR) continue; /* skip this request */
            GET_ONE_COUNT(count)
            /* when bufcounts == NULL, it means the same as counts[] */
            if (bufcounts == NULL) buflen = 1;

            err = ncmpii_igetput_varm(ncp, varp, starts[i], count, NULL, NULL,
                                      bufs[i], buflen, datatypes[i],
                                      &req_ids[i], rw_flag, 0, 0);
            if (status == NC_NOERR) status = err;
            NCI_Free(count);
        } else if (strides == NULL) { /* vara */
            /* check whether starts[i] and counts[i] are valid */
            err = NC_start_count_stride_ck(ncp, varp, starts[i], counts[i], NULL, rw_flag);
            if (status == NC_NOERR) status = err;
            if (err != NC_NOERR) continue; /* skip this request */
            /* when bufcounts == NULL, it means the same as counts[] */
            if (bufcounts == NULL) {
                for (buflen=1, j=0; j<varp->ndims; j++)
                    buflen *= counts[i][j];
            }
            err = ncmpii_igetput_varm(ncp, varp, starts[i], counts[i], NULL,
                                      NULL, bufs[i], buflen, datatypes[i],
                                      &req_ids[i], rw_flag, 0, 0);
            if (status == NC_NOERR) status = err;
        } else if (imaps == NULL) {   /* vars */
            /* check whether starts[i], counts[i], and strides[i] are valid */
            err = NC_start_count_stride_ck(ncp, varp, starts[i], counts[i], strides[i], rw_flag);
            if (status == NC_NOERR) status = err;
            if (err != NC_NOERR) continue; /* skip this request */
            /* when bufcounts == NULL, it means the same as counts[] */
            if (bufcounts == NULL) {
                for (buflen=1, j=0; j<varp->ndims; j++)
                    buflen *= counts[i][j];
            }
            err = ncmpii_igetput_varm(ncp, varp, starts[i], counts[i],
                                      strides[i], NULL, bufs[i], buflen,
                                      datatypes[i], &req_ids[i],
                                      rw_flag, 0, 0);
            if (status == NC_NOERR) status = err;
        } else {                      /* varm */
            /* check whether starts[i], counts[i], and strides[i] are valid */
            err = NC_start_count_stride_ck(ncp, varp, starts[i], counts[i], strides[i], rw_flag);
            if (status == NC_NOERR) status = err;
            if (err != NC_NOERR) continue; /* skip this request */
            /* when bufcounts == NULL, it means the same as counts[] */
            if (bufcounts == NULL) {
                for (buflen=1, j=0; j<varp->ndims; j++)
                    buflen *= counts[i][j];
            }
            err = ncmpii_igetput_varm(ncp, varp, starts[i], counts[i],
                                      strides[i], imaps[i], bufs[i],
                                      buflen, datatypes[i],
                                      &req_ids[i], rw_flag, 0, 0);
            if (status == NC_NOERR) status = err;
        }
    }

    if (io_method == COLL_IO)
        err = ncmpi_wait_all(ncid, num, req_ids, statuses);
    else
        err = ncmpi_wait(ncid, num, req_ids, statuses);

    /* return the first error if there is one */
    if (status == NC_NOERR) status = err;
    if (status == NC_NOERR) {
        for (i=0; i<num; i++)
            if (statuses[i] != NC_NOERR) {
                status = statuses[i];
                break;
            }
    }

    if (num > 0) NCI_Free(req_ids);

    return status;
}
